@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,11 +28,11 @@ template <typename T, bool CONJ = false>
 void testing_dot_batched_bad_arg(const Arguments& arg)
 {
     auto rocblas_dot_batched_fn
-        = arg.api == FORTRAN
+        = arg.api & c_API_FORTRAN
               ? (CONJ ? rocblas_dotc_batched<T, true> : rocblas_dot_batched<T, true>)
               : (CONJ ? rocblas_dotc_batched<T, false> : rocblas_dot_batched<T, false>);
     auto rocblas_dot_batched_fn_64
-        = arg.api == FORTRAN_64
+        = arg.api & c_API_FORTRAN
               ? (CONJ ? rocblas_dotc_batched_64<T, true> : rocblas_dot_batched_64<T, true>)
               : (CONJ ? rocblas_dotc_batched_64<T, false> : rocblas_dot_batched_64<T, false>);
 
@@ -82,21 +82,15 @@ void testing_dot_batched_bad_arg(const Arguments& arg)
     }
 }
 
-template <typename T>
-void testing_dotc_batched_bad_arg(const Arguments& arg)
-{
-    testing_dot_batched_bad_arg<T, true>(arg);
-}
-
 template <typename T, bool CONJ = false>
 void testing_dot_batched(const Arguments& arg)
 {
     auto rocblas_dot_batched_fn
-        = arg.api == FORTRAN
+        = arg.api & c_API_FORTRAN
               ? (CONJ ? rocblas_dotc_batched<T, true> : rocblas_dot_batched<T, true>)
               : (CONJ ? rocblas_dotc_batched<T, false> : rocblas_dot_batched<T, false>);
     auto rocblas_dot_batched_fn_64
-        = arg.api == FORTRAN_64
+        = arg.api & c_API_FORTRAN
               ? (CONJ ? rocblas_dotc_batched_64<T, true> : rocblas_dot_batched_64<T, true>)
               : (CONJ ? rocblas_dotc_batched_64<T, false> : rocblas_dot_batched_64<T, false>);
 
@@ -206,13 +200,78 @@ void testing_dot_batched(const Arguments& arg)
                         batch_count,
                         d_rocblas_result_device));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                CHECK_HIP_ERROR(rocblas_result_device.transfer_from(d_rocblas_result_device));
+                host_vector<T> rocblas_result_device_copy(batch_count);
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_batch_vector<T> dx_copy(N, incx, batch_count);
+                    device_batch_vector<T> dy_copy(N, incy, batch_count);
+                    device_vector<T>       d_rocblas_result_device_copy(batch_count);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_rocblas_result_device_copy.memcheck());
+
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_HIP_ERROR(dy_copy.transfer_from(hy));
+
+                    // arg.algo indicates to force optimized x dot x kernel algorithm with equal inc
+                    auto dy_ptr_copy
+                        = (arg.algo) ? dx_copy.ptr_on_device() : dy_copy.ptr_on_device();
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        DAPI_CHECK(rocblas_dot_batched_fn,
+                                   (handle_copy,
+                                    N,
+                                    dx_copy.ptr_on_device(),
+                                    incx,
+                                    dy_ptr_copy,
+                                    incy,
+                                    batch_count,
+                                    d_rocblas_result_device_copy));
+
+                        CHECK_HIP_ERROR(
+                            rocblas_result_device_copy.transfer_from(d_rocblas_result_device_copy));
+
+                        unit_check_general<T>(1,
+                                              1,
+                                              1,
+                                              1,
+                                              rocblas_result_device,
+                                              rocblas_result_device_copy,
+                                              batch_count);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
         for(size_t b = 0; b < batch_count; ++b)
         {
-            (CONJ ? cblas_dotc<T> : cblas_dot<T>)(N, hx[b], incx, hy_ptr[b], incy, &cpu_result[b]);
+            (CONJ ? ref_dotc<T, T>
+                  : ref_dot<T, T>)(N, hx[b], incx, hy_ptr[b], incy, &cpu_result[b]);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
@@ -314,9 +373,3 @@ void testing_dot_batched(const Arguments& arg)
             rocblas_error_device);
     }
 }
-
-template <typename T>
-void testing_dotc_batched(const Arguments& arg)
-{
-    testing_dot_batched<T, true>(arg);
-}
