@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,9 +27,9 @@
 template <typename T>
 void testing_nrm2_bad_arg(const Arguments& arg)
 {
-    auto rocblas_nrm2_fn = arg.api == FORTRAN ? rocblas_nrm2<T, true> : rocblas_nrm2<T, false>;
+    auto rocblas_nrm2_fn = arg.api & c_API_FORTRAN ? rocblas_nrm2<T, true> : rocblas_nrm2<T, false>;
     auto rocblas_nrm2_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_nrm2_64<T, true> : rocblas_nrm2_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_nrm2_64<T, true> : rocblas_nrm2_64<T, false>;
 
     int64_t             N         = 100;
     int64_t             incx      = 1;
@@ -58,15 +58,15 @@ void testing_nrm2_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_nrm2(const Arguments& arg)
 {
-    auto rocblas_nrm2_fn = arg.api == FORTRAN ? rocblas_nrm2<T, true> : rocblas_nrm2<T, false>;
+    auto rocblas_nrm2_fn = arg.api & c_API_FORTRAN ? rocblas_nrm2<T, true> : rocblas_nrm2<T, false>;
     auto rocblas_nrm2_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_nrm2_64<T, true> : rocblas_nrm2_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_nrm2_64<T, true> : rocblas_nrm2_64<T, false>;
 
     int64_t N    = arg.N;
     int64_t incx = arg.incx;
 
-    double rocblas_error_1;
-    double rocblas_error_2;
+    double error_host_ptr;
+    double error_device_ptr;
 
     rocblas_local_handle handle{arg};
 
@@ -103,17 +103,16 @@ void testing_nrm2(const Arguments& arg)
     // Naming: `h` is in CPU (host) memory(eg hx), `d` is in GPU (device) memory (eg dx).
     // Allocate host memory
     host_vector<T>         hx(N, incx);
-    host_vector<real_t<T>> rocblas_result_1(1, 1);
-    host_vector<real_t<T>> rocblas_result_2(1, 1);
+    host_vector<real_t<T>> rocblas_result(1, 1);
     host_vector<real_t<T>> cpu_result(1, 1);
 
     // Allocate device memory
     device_vector<T>         dx(N, incx);
-    device_vector<real_t<T>> d_rocblas_result_2(1);
+    device_vector<real_t<T>> d_rocblas_result(1);
 
     // Check device memory allocation
     CHECK_DEVICE_ALLOCATION(dx.memcheck());
-    CHECK_DEVICE_ALLOCATION(d_rocblas_result_2.memcheck());
+    CHECK_DEVICE_ALLOCATION(d_rocblas_result.memcheck());
 
     // Initial Data on CPU
     rocblas_init_vector(hx, arg, rocblas_client_alpha_sets_nan, true, true);
@@ -128,71 +127,93 @@ void testing_nrm2(const Arguments& arg)
         if(arg.pointer_mode_host)
         {
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
-            DAPI_CHECK(rocblas_nrm2_fn, (handle, N, dx, incx, rocblas_result_1));
+            DAPI_CHECK(rocblas_nrm2_fn, (handle, N, dx, incx, rocblas_result));
         }
 
         if(arg.pointer_mode_device)
         {
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
             handle.pre_test(arg);
-            DAPI_CHECK(rocblas_nrm2_fn, (handle, N, dx, incx, d_rocblas_result_2));
+            DAPI_CHECK(rocblas_nrm2_fn, (handle, N, dx, incx, d_rocblas_result));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                host_vector<real_t<T>> rocblas_result_copy(1, 1);
+                CHECK_HIP_ERROR(rocblas_result.transfer_from(d_rocblas_result));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    // Allocate device memory in new device
+                    device_vector<T>         dx_copy(N, incx);
+                    device_vector<real_t<T>> d_rocblas_result_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_rocblas_result_copy.memcheck());
+
+                    // copy data from CPU to device
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        DAPI_CHECK(rocblas_nrm2_fn,
+                                   (handle_copy, N, dx_copy, incx, d_rocblas_result_copy));
+                        CHECK_HIP_ERROR(rocblas_result_copy.transfer_from(d_rocblas_result_copy));
+                        unit_check_general<real_t<T>, real_t<T>>(
+                            1, 1, 1, rocblas_result, rocblas_result_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
-        cblas_nrm2<T>(N, hx, incx, cpu_result);
+        ref_nrm2<T>(N, hx, incx, cpu_result);
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
-        real_t<T> abs_result = cpu_result[0] > 0 ? cpu_result[0] : -cpu_result[0];
-        real_t<T> abs_error;
-        if(abs_result > 0)
-        {
-            abs_error = std::numeric_limits<real_t<T>>::epsilon() * N * abs_result;
-        }
-        else
-        {
-            abs_error = std::numeric_limits<real_t<T>>::epsilon() * N;
-        }
-        real_t<T> tolerance = 2.0; //  accounts for rounding in reduction sum. depends on n.
-            //  If test fails, try decreasing n or increasing tolerance.
-        abs_error *= tolerance;
-
-        if(arg.pointer_mode_host)
-        {
+        auto compare_to_gold = [&] {
             if(!rocblas_isnan(arg.alpha))
             {
                 if(arg.unit_check)
                 {
+                    double abs_error = sum_near_tolerance<T>(N, cpu_result[0]);
                     near_check_general<real_t<T>, real_t<T>>(
-                        1, 1, 1, cpu_result, rocblas_result_1, abs_error);
+                        1, 1, 1, cpu_result, rocblas_result, abs_error);
                 }
             }
 
+            double error = 0.0;
             if(arg.norm_check)
             {
-                rocblas_error_1
-                    = rocblas_abs((cpu_result[0] - rocblas_result_1[0]) / cpu_result[0]);
+                error = rocblas_abs((cpu_result[0] - rocblas_result[0]) / cpu_result[0]);
             }
+            return error;
+        };
+
+        if(arg.pointer_mode_host)
+        {
+            error_host_ptr = compare_to_gold();
         }
 
         if(arg.pointer_mode_device)
         {
-            CHECK_HIP_ERROR(rocblas_result_2.transfer_from(d_rocblas_result_2));
-            if(!rocblas_isnan(arg.alpha))
-            {
-                if(arg.unit_check)
-                {
-                    near_check_general<real_t<T>, real_t<T>>(
-                        1, 1, 1, cpu_result, rocblas_result_2, abs_error);
-                }
-            }
-
-            if(arg.norm_check)
-            {
-                rocblas_error_2
-                    = rocblas_abs((cpu_result[0] - rocblas_result_2[0]) / cpu_result[0]);
-            }
+            CHECK_HIP_ERROR(rocblas_result.transfer_from(d_rocblas_result));
+            error_device_ptr = compare_to_gold();
         }
     }
 
@@ -211,7 +232,7 @@ void testing_nrm2(const Arguments& arg)
             if(iter == number_cold_calls)
                 gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-            DAPI_DISPATCH(rocblas_nrm2_fn, (handle, N, dx, incx, d_rocblas_result_2));
+            DAPI_DISPATCH(rocblas_nrm2_fn, (handle, N, dx, incx, d_rocblas_result));
         }
 
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
@@ -222,7 +243,7 @@ void testing_nrm2(const Arguments& arg)
                                                  nrm2_gflop_count<T>(N),
                                                  nrm2_gbyte_count<T>(N),
                                                  cpu_time_used,
-                                                 rocblas_error_1,
-                                                 rocblas_error_2);
+                                                 error_host_ptr,
+                                                 error_device_ptr);
     }
 }
