@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,9 +27,10 @@
 template <typename T>
 void testing_rotmg_bad_arg(const Arguments& arg)
 {
-    auto rocblas_rotmg_fn = arg.api == FORTRAN ? rocblas_rotmg<T, true> : rocblas_rotmg<T, false>;
+    auto rocblas_rotmg_fn
+        = arg.api & c_API_FORTRAN ? rocblas_rotmg<T, true> : rocblas_rotmg<T, false>;
     auto rocblas_rotmg_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_rotmg_64<T, true> : rocblas_rotmg_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_rotmg_64<T, true> : rocblas_rotmg_64<T, false>;
 
     rocblas_local_handle handle{arg};
 
@@ -63,9 +64,10 @@ void testing_rotmg_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_rotmg(const Arguments& arg)
 {
-    auto rocblas_rotmg_fn = arg.api == FORTRAN ? rocblas_rotmg<T, true> : rocblas_rotmg<T, false>;
+    auto rocblas_rotmg_fn
+        = arg.api & c_API_FORTRAN ? rocblas_rotmg<T, true> : rocblas_rotmg<T, false>;
     auto rocblas_rotmg_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_rotmg_64<T, true> : rocblas_rotmg_64<T, false>;
+        = arg.api & c_API_FORTRAN ? rocblas_rotmg_64<T, true> : rocblas_rotmg_64<T, false>;
 
     rocblas_local_handle handle{arg};
 
@@ -89,11 +91,11 @@ void testing_rotmg(const Arguments& arg)
             host_vector<T> hparams_gold = params;
 
             cpu_time_used = get_time_us_no_sync();
-            cblas_rotmg<T>(&hparams_gold[0],
-                           &hparams_gold[1],
-                           &hparams_gold[2],
-                           &hparams_gold[3],
-                           &hparams_gold[4]);
+            ref_rotmg<T>(&hparams_gold[0],
+                         &hparams_gold[1],
+                         &hparams_gold[2],
+                         &hparams_gold[3],
+                         &hparams_gold[4]);
             cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
             if(arg.pointer_mode_host)
@@ -137,6 +139,48 @@ void testing_rotmg(const Arguments& arg)
 
                 CHECK_HIP_ERROR(hparams.transfer_from(dparams));
 
+                if(arg.repeatability_check)
+                {
+                    host_vector<T> hparams_copy(9, 1);
+
+                    // multi-GPU support
+                    int device_id, device_count;
+                    CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                    for(int dev_id = 0; dev_id < device_count; dev_id++)
+                    {
+                        CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                        if(device_id != dev_id)
+                            CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                        //New rocblas handle for new device
+                        rocblas_local_handle handle_copy{arg};
+
+                        // Allocate device memory in new device
+                        device_vector<T> dparams_copy(9, 1);
+
+                        // Check device memory allocation
+                        CHECK_DEVICE_ALLOCATION(dparams_copy.memcheck());
+
+                        CHECK_ROCBLAS_ERROR(
+                            rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                        for(int runs = 0; runs < arg.iters; runs++)
+                        {
+                            CHECK_HIP_ERROR(dparams_copy.transfer_from(params));
+                            DAPI_CHECK(rocblas_rotmg_fn,
+                                       (handle_copy,
+                                        dparams_copy,
+                                        dparams_copy + 1,
+                                        dparams_copy + 2,
+                                        dparams_copy + 3,
+                                        dparams_copy + 4));
+                            CHECK_HIP_ERROR(hparams_copy.transfer_from(dparams_copy));
+                            unit_check_general<T>(1, 9, 1, hparams, hparams_copy);
+                        }
+                    }
+                    return;
+                }
+
                 if(arg.unit_check)
                     near_check_general<T>(1, 9, 1, hparams_gold, hparams, rel_error);
 
