@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,26 +23,14 @@
 
 #pragma once
 
-#include "bytes.hpp"
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "near.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 template <typename T>
 void testing_gemv_bad_arg(const Arguments& arg)
 {
-    auto rocblas_gemv_fn = arg.api == FORTRAN ? rocblas_gemv<T, true> : rocblas_gemv<T, false>;
+    auto rocblas_gemv_fn = arg.api & c_API_FORTRAN ? rocblas_gemv<T, true> : rocblas_gemv<T, false>;
+    auto rocblas_gemv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_gemv_64<T, true> : rocblas_gemv_64<T, false>;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
@@ -50,11 +38,11 @@ void testing_gemv_bad_arg(const Arguments& arg)
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, pointer_mode));
 
         const rocblas_operation transA = rocblas_operation_none;
-        const rocblas_int       M      = 100;
-        const rocblas_int       N      = 100;
-        const rocblas_int       lda    = 100;
-        const rocblas_int       incx   = 1;
-        const rocblas_int       incy   = 1;
+        const int64_t           M      = 100;
+        const int64_t           N      = 100;
+        const int64_t           lda    = 100;
+        const int64_t           incx   = 1;
+        const int64_t           incy   = 1;
 
         device_vector<T> alpha_d(1), beta_d(1), zero_d(1), one_d(1);
         const T          alpha_h(1), beta_h(1), zero_h(0), one_h(1);
@@ -103,86 +91,80 @@ void testing_gemv_bad_arg(const Arguments& arg)
         CHECK_HIP_ERROR(dx.transfer_from(hx));
         CHECK_HIP_ERROR(dy.transfer_from(hy));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_gemv_fn(nullptr, transA, M, N, alpha, dA, lda, dx, incx, beta, dy, incy),
-            rocblas_status_invalid_handle);
+        DAPI_EXPECT(rocblas_status_invalid_handle,
+                    rocblas_gemv_fn,
+                    (nullptr, transA, M, N, alpha, dA, lda, dx, incx, beta, dy, incy));
 
-        EXPECT_ROCBLAS_STATUS(rocblas_gemv_fn(handle,
-                                              (rocblas_operation)rocblas_fill_full,
-                                              M,
-                                              N,
-                                              alpha,
-                                              dA,
-                                              lda,
-                                              dx,
-                                              incx,
-                                              beta,
-                                              dy,
-                                              incy),
-                              rocblas_status_invalid_value);
+        DAPI_EXPECT(rocblas_status_invalid_value,
+                    rocblas_gemv_fn,
+                    (handle,
+                     (rocblas_operation)rocblas_fill_full,
+                     M,
+                     N,
+                     alpha,
+                     dA,
+                     lda,
+                     dx,
+                     incx,
+                     beta,
+                     dy,
+                     incy));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_gemv_fn(handle, transA, M, N, nullptr, dA, lda, dx, incx, beta, dy, incy),
-            rocblas_status_invalid_pointer);
+        DAPI_EXPECT(rocblas_status_invalid_pointer,
+                    rocblas_gemv_fn,
+                    (handle, transA, M, N, nullptr, dA, lda, dx, incx, beta, dy, incy));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_gemv_fn(handle, transA, M, N, alpha, dA, lda, dx, incx, nullptr, dy, incy),
-            rocblas_status_invalid_pointer);
+        DAPI_EXPECT(rocblas_status_invalid_pointer,
+                    rocblas_gemv_fn,
+                    (handle, transA, M, N, alpha, dA, lda, dx, incx, nullptr, dy, incy));
 
         if(pointer_mode == rocblas_pointer_mode_host)
         {
-            EXPECT_ROCBLAS_STATUS(
-                rocblas_gemv_fn(
-                    handle, transA, M, N, alpha, nullptr, lda, dx, incx, beta, dy, incy),
-                rocblas_status_invalid_pointer);
+            DAPI_EXPECT(rocblas_status_invalid_pointer,
+                        rocblas_gemv_fn,
+                        (handle, transA, M, N, alpha, nullptr, lda, dx, incx, beta, dy, incy));
 
-            EXPECT_ROCBLAS_STATUS(
-                rocblas_gemv_fn(
-                    handle, transA, M, N, alpha, dA, lda, nullptr, incx, beta, dy, incy),
-                rocblas_status_invalid_pointer);
+            DAPI_EXPECT(rocblas_status_invalid_pointer,
+                        rocblas_gemv_fn,
+                        (handle, transA, M, N, alpha, dA, lda, nullptr, incx, beta, dy, incy));
 
-            EXPECT_ROCBLAS_STATUS(
-                rocblas_gemv_fn(
-                    handle, transA, M, N, alpha, dA, lda, dx, incx, beta, nullptr, incy),
-                rocblas_status_invalid_pointer);
+            DAPI_EXPECT(rocblas_status_invalid_pointer,
+                        rocblas_gemv_fn,
+                        (handle, transA, M, N, alpha, dA, lda, dx, incx, beta, nullptr, incy));
         }
 
         // If M==0, then all pointers may be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_gemv_fn(
-                handle, transA, 0, N, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy),
-            rocblas_status_success);
+        DAPI_CHECK(
+            rocblas_gemv_fn,
+            (handle, transA, 0, N, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy));
 
         // If N==0, then all pointers may be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_gemv_fn(
-                handle, transA, M, 0, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy),
-            rocblas_status_success);
+        DAPI_CHECK(
+            rocblas_gemv_fn,
+            (handle, transA, M, 0, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy));
 
         // If alpha==0, then A and X may be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_gemv_fn(
-                handle, transA, M, N, zero, nullptr, lda, nullptr, incx, beta, dy, incy),
-            rocblas_status_success);
+        DAPI_CHECK(rocblas_gemv_fn,
+                   (handle, transA, M, N, zero, nullptr, lda, nullptr, incx, beta, dy, incy));
 
         // If alpha==0 && beta==1, then A, X and Y may be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_gemv_fn(
-                handle, transA, M, N, zero, nullptr, lda, nullptr, incx, one, nullptr, incy),
-            rocblas_status_success);
+        DAPI_CHECK(rocblas_gemv_fn,
+                   (handle, transA, M, N, zero, nullptr, lda, nullptr, incx, one, nullptr, incy));
     }
 }
 
 template <typename T>
 void testing_gemv(const Arguments& arg)
 {
-    auto rocblas_gemv_fn = arg.api == FORTRAN ? rocblas_gemv<T, true> : rocblas_gemv<T, false>;
+    auto rocblas_gemv_fn = arg.api & c_API_FORTRAN ? rocblas_gemv<T, true> : rocblas_gemv<T, false>;
+    auto rocblas_gemv_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_gemv_64<T, true> : rocblas_gemv_64<T, false>;
 
-    rocblas_int       M       = arg.M;
-    rocblas_int       N       = arg.N;
-    rocblas_int       lda     = arg.lda;
-    rocblas_int       incx    = arg.incx;
-    rocblas_int       incy    = arg.incy;
+    int64_t           M       = arg.M;
+    int64_t           N       = arg.N;
+    int64_t           lda     = arg.lda;
+    int64_t           incx    = arg.incx;
+    int64_t           incy    = arg.incy;
     T                 h_alpha = arg.get_alpha<T>();
     T                 h_beta  = arg.get_beta<T>();
     rocblas_operation transA  = char2rocblas_operation(arg.transA);
@@ -194,10 +176,10 @@ void testing_gemv(const Arguments& arg)
     bool invalid_size = M < 0 || N < 0 || lda < M || lda < 1 || !incx || !incy;
     if(invalid_size || !M || !N)
     {
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_gemv_fn(
-                handle, transA, M, N, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy),
-            invalid_size ? rocblas_status_invalid_size : rocblas_status_success);
+        DAPI_EXPECT(
+            invalid_size ? rocblas_status_invalid_size : rocblas_status_success,
+            rocblas_gemv_fn,
+            (handle, transA, M, N, nullptr, nullptr, lda, nullptr, incx, nullptr, nullptr, incy));
 
         return;
     }
@@ -256,9 +238,9 @@ void testing_gemv(const Arguments& arg)
     CHECK_HIP_ERROR(dx.transfer_from(hx));
     CHECK_HIP_ERROR(dy.transfer_from(hy));
 
-    double gpu_time_used, cpu_time_used;
-    double rocblas_error_1;
-    double rocblas_error_2;
+    double cpu_time_used;
+    double error_host;
+    double error_device;
 
     /* =====================================================================
            ROCBLAS
@@ -269,8 +251,8 @@ void testing_gemv(const Arguments& arg)
         {
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
             handle.pre_test(arg);
-            CHECK_ROCBLAS_ERROR(rocblas_gemv_fn(
-                handle, transA, M, N, &h_alpha, dA, lda, dx, incx, &h_beta, dy, incy));
+            DAPI_CHECK(rocblas_gemv_fn,
+                       (handle, transA, M, N, &h_alpha, dA, lda, dx, incx, &h_beta, dy, incy));
             handle.post_test(arg);
 
             CHECK_HIP_ERROR(hy.transfer_from(dy));
@@ -284,55 +266,131 @@ void testing_gemv(const Arguments& arg)
 
             CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_device));
             handle.pre_test(arg);
-            CHECK_ROCBLAS_ERROR(rocblas_gemv_fn(
-                handle, transA, M, N, d_alpha, dA, lda, dx, incx, d_beta, dy, incy));
+            DAPI_CHECK(rocblas_gemv_fn,
+                       (handle, transA, M, N, d_alpha, dA, lda, dx, incx, d_beta, dy, incy));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                //Transfer original results from device to host
+                CHECK_HIP_ERROR(hy.transfer_from(dy));
+                //Host buffer to store results subsequent iterations
+                host_vector<T> hy_copy(dim_y, incy);
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+                    //Allocate device memory in new device
+                    device_vector<T> dy_copy(dim_y, incy, HMM);
+                    device_matrix<T> dA_copy(M, N, lda, HMM);
+                    device_vector<T> dx_copy(dim_x, incx, HMM);
+                    device_vector<T> d_alpha_copy(1, 1, HMM);
+                    device_vector<T> d_beta_copy(1, 1, HMM);
+
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_alpha_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_beta_copy.memcheck());
+
+                    CHECK_HIP_ERROR(dA_copy.transfer_from(hA));
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_HIP_ERROR(d_alpha_copy.transfer_from(halpha));
+                    CHECK_HIP_ERROR(d_beta_copy.transfer_from(hbeta));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        CHECK_HIP_ERROR(dy_copy.transfer_from(hy_gold));
+                        DAPI_CHECK(rocblas_gemv_fn,
+                                   (handle_copy,
+                                    transA,
+                                    M,
+                                    N,
+                                    d_alpha_copy,
+                                    dA_copy,
+                                    lda,
+                                    dx_copy,
+                                    incx,
+                                    d_beta_copy,
+                                    dy_copy,
+                                    incy));
+
+                        CHECK_HIP_ERROR(hy_copy.transfer_from(dy_copy));
+                        unit_check_general<T>(1, dim_y, incy, hy, hy_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
 
-        cblas_gemv<T>(transA, M, N, h_alpha, (T*)hA, lda, (T*)hx, incx, h_beta, (T*)hy_gold, incy);
+        ref_gemv<T>(transA, M, N, h_alpha, (T*)hA, lda, (T*)hx, incx, h_beta, (T*)hy_gold, incy);
 
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
+        auto compare_hy_to_gold = [&] {
+            if(arg.unit_check)
+            {
+                bool use_near = reduction_requires_near<T>(arg, dim_x);
+                if(use_near)
+                {
+                    const double tol = dim_x * sum_error_tolerance<T>;
+                    near_check_general<T>(1, dim_y, incy, hy_gold, hy, tol);
+                }
+                else
+                {
+                    unit_check_general<T>(1, dim_y, incy, hy_gold, hy);
+                }
+            }
+            double error = 0;
+            if(arg.norm_check)
+                error = norm_check_general<T>('F', 1, dim_y, incy, hy_gold, hy);
+            return error;
+        };
+
         if(arg.pointer_mode_host)
         {
-            if(arg.unit_check)
-                unit_check_general<T>(1, dim_y, incy, hy_gold, hy);
-            if(arg.norm_check)
-                rocblas_error_1 = norm_check_general<T>('F', 1, dim_y, incy, hy_gold, hy);
+            error_host = compare_hy_to_gold();
         }
 
         if(arg.pointer_mode_device)
         {
             CHECK_HIP_ERROR(hy.transfer_from(dy));
-            if(arg.unit_check)
-                unit_check_general<T>(1, dim_y, incy, hy_gold, hy);
-            if(arg.norm_check)
-                rocblas_error_2 = norm_check_general<T>('F', 1, dim_y, incy, hy_gold, hy);
+            error_device = compare_hy_to_gold();
         }
     }
 
     if(arg.timing)
     {
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
+        double gpu_time_used;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
 
-        for(int iter = 0; iter < number_cold_calls; iter++)
-        {
-            rocblas_gemv_fn(handle, transA, M, N, &h_alpha, dA, lda, dx, incx, &h_beta, dy, incy);
-        }
-
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-        gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-        for(int iter = 0; iter < number_hot_calls; iter++)
+        for(int iter = 0; iter < total_calls; iter++)
         {
-            rocblas_gemv_fn(handle, transA, M, N, &h_alpha, dA, lda, dx, incx, &h_beta, dy, incy);
+            if(iter == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream);
+
+            DAPI_DISPATCH(rocblas_gemv_fn,
+                          (handle, transA, M, N, &h_alpha, dA, lda, dx, incx, &h_beta, dy, incy));
         }
 
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
@@ -344,7 +402,7 @@ void testing_gemv(const Arguments& arg)
             gemv_gflop_count<T>(transA, M, N),
             gemv_gbyte_count<T>(transA, M, N),
             cpu_time_used,
-            rocblas_error_1,
-            rocblas_error_2);
+            error_host,
+            error_device);
     }
 }
