@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,21 +22,10 @@
 
 #pragma once
 
-#include "cblas_interface.hpp"
-#include "flops.hpp"
-#include "norm.hpp"
-#include "rocblas.hpp"
-#include "rocblas_datatype2string.hpp"
-#include "rocblas_init.hpp"
-#include "rocblas_math.hpp"
-#include "rocblas_matrix.hpp"
-#include "rocblas_random.hpp"
-#include "rocblas_test.hpp"
-#include "rocblas_vector.hpp"
-#include "unit.hpp"
-#include "utility.hpp"
+#include "testing_common.hpp"
 
 #include "blas3/rocblas_trsm.hpp"
+#include "src64/blas3/rocblas_trsm_64.hpp"
 
 #define ERROR_EPS_MULTIPLIER 40
 #define RESIDUAL_EPS_MULTIPLIER 40
@@ -44,17 +33,19 @@
 template <typename T>
 void testing_trsm_bad_arg(const Arguments& arg)
 {
-    auto rocblas_trsm_fn = arg.api == FORTRAN ? rocblas_trsm<T, true> : rocblas_trsm<T, false>;
+    auto rocblas_trsm_fn = arg.api & c_API_FORTRAN ? rocblas_trsm<T, true> : rocblas_trsm<T, false>;
+    auto rocblas_trsm_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_trsm_64<T, true> : rocblas_trsm_64<T, false>;
 
     for(auto pointer_mode : {rocblas_pointer_mode_host, rocblas_pointer_mode_device})
     {
         rocblas_local_handle handle{arg};
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, pointer_mode));
 
-        const rocblas_int M   = 100;
-        const rocblas_int N   = 100;
-        const rocblas_int lda = 100;
-        const rocblas_int ldb = 100;
+        const int64_t M   = 100;
+        const int64_t N   = 100;
+        const int64_t lda = 100;
+        const int64_t ldb = 100;
 
         device_vector<T> alpha_d(1), zero_d(1);
 
@@ -76,7 +67,7 @@ void testing_trsm_bad_arg(const Arguments& arg)
         const rocblas_operation transA = rocblas_operation_none;
         const rocblas_diagonal  diag   = rocblas_diagonal_non_unit;
 
-        rocblas_int K = side == rocblas_side_left ? M : N;
+        int64_t K = side == rocblas_side_left ? M : N;
 
         // Allocate device memory
         device_matrix<T> dA(K, K, lda);
@@ -87,126 +78,181 @@ void testing_trsm_bad_arg(const Arguments& arg)
         CHECK_DEVICE_ALLOCATION(dB.memcheck());
 
         // check for invalid enum
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(
-                handle, rocblas_side_both, uplo, transA, diag, M, N, alpha, dA, lda, dB, ldb),
-            rocblas_status_invalid_value);
+        DAPI_EXPECT(rocblas_status_invalid_value,
+                    rocblas_trsm_fn,
+                    (handle, rocblas_side_both, uplo, transA, diag, M, N, alpha, dA, lda, dB, ldb));
 
-        EXPECT_ROCBLAS_STATUS(rocblas_trsm_fn(handle,
-                                              side,
-                                              (rocblas_fill)rocblas_side_both,
-                                              transA,
-                                              diag,
-                                              M,
-                                              N,
-                                              alpha,
-                                              dA,
-                                              lda,
-                                              dB,
-                                              ldb),
-                              rocblas_status_invalid_value);
+        DAPI_EXPECT(rocblas_status_invalid_value,
+                    rocblas_trsm_fn,
+                    (handle,
+                     side,
+                     (rocblas_fill)rocblas_side_both,
+                     transA,
+                     diag,
+                     M,
+                     N,
+                     alpha,
+                     dA,
+                     lda,
+                     dB,
+                     ldb));
 
-        EXPECT_ROCBLAS_STATUS(rocblas_trsm_fn(handle,
-                                              side,
-                                              uplo,
-                                              (rocblas_operation)rocblas_side_both,
-                                              diag,
-                                              M,
-                                              N,
-                                              alpha,
-                                              dA,
-                                              lda,
-                                              dB,
-                                              ldb),
-                              rocblas_status_invalid_value);
+        DAPI_EXPECT(rocblas_status_invalid_value,
+                    rocblas_trsm_fn,
+                    (handle,
+                     side,
+                     uplo,
+                     (rocblas_operation)rocblas_side_both,
+                     diag,
+                     M,
+                     N,
+                     alpha,
+                     dA,
+                     lda,
+                     dB,
+                     ldb));
 
-        EXPECT_ROCBLAS_STATUS(rocblas_trsm_fn(handle,
-                                              side,
-                                              uplo,
-                                              transA,
-                                              (rocblas_diagonal)rocblas_side_both,
-                                              M,
-                                              N,
-                                              alpha,
-                                              dA,
-                                              lda,
-                                              dB,
-                                              ldb),
-                              rocblas_status_invalid_value);
+        DAPI_EXPECT(rocblas_status_invalid_value,
+                    rocblas_trsm_fn,
+                    (handle,
+                     side,
+                     uplo,
+                     transA,
+                     (rocblas_diagonal)rocblas_side_both,
+                     M,
+                     N,
+                     alpha,
+                     dA,
+                     lda,
+                     dB,
+                     ldb));
 
         // check for invalid size
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(handle, side, uplo, transA, diag, -1, N, alpha, dA, lda, dB, ldb),
-            rocblas_status_invalid_size);
+        DAPI_EXPECT(rocblas_status_invalid_size,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, -1, N, alpha, dA, lda, dB, ldb));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(handle, side, uplo, transA, diag, M, -1, alpha, dA, lda, dB, ldb),
-            rocblas_status_invalid_size);
+        DAPI_EXPECT(rocblas_status_invalid_size,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, M, -1, alpha, dA, lda, dB, ldb));
 
         /// check for invalid leading dimension
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(handle, side, uplo, transA, diag, M, N, alpha, dA, lda, dB, M - 1),
-            rocblas_status_invalid_size);
+        DAPI_EXPECT(rocblas_status_invalid_size,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, M, N, alpha, dA, lda, dB, M - 1));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(
-                handle, rocblas_side_left, uplo, transA, diag, M, N, alpha, dA, M - 1, dB, ldb),
-            rocblas_status_invalid_size);
+        DAPI_EXPECT(
+            rocblas_status_invalid_size,
+            rocblas_trsm_fn,
+            (handle, rocblas_side_left, uplo, transA, diag, M, N, alpha, dA, M - 1, dB, ldb));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(
-                handle, rocblas_side_right, uplo, transA, diag, M, N, alpha, dA, N - 1, dB, ldb),
-            rocblas_status_invalid_size);
+        DAPI_EXPECT(
+            rocblas_status_invalid_size,
+            rocblas_trsm_fn,
+            (handle, rocblas_side_right, uplo, transA, diag, M, N, alpha, dA, N - 1, dB, ldb));
 
         // check that nullpointer gives rocblas_status_invalid_handle or rocblas_status_invalid_pointer
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(nullptr, side, uplo, transA, diag, M, N, alpha, dA, lda, dB, ldb),
-            rocblas_status_invalid_handle);
+        DAPI_EXPECT(rocblas_status_invalid_handle,
+                    rocblas_trsm_fn,
+                    (nullptr, side, uplo, transA, diag, M, N, alpha, dA, lda, dB, ldb));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(handle, side, uplo, transA, diag, M, N, nullptr, dA, lda, dB, ldb),
-            rocblas_status_invalid_pointer);
+        DAPI_EXPECT(rocblas_status_invalid_pointer,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, M, N, nullptr, dA, lda, dB, ldb));
 
         if(pointer_mode == rocblas_pointer_mode_host)
         {
-            EXPECT_ROCBLAS_STATUS(
-                rocblas_trsm_fn(
-                    handle, side, uplo, transA, diag, M, N, alpha, nullptr, lda, dB, ldb),
-                rocblas_status_invalid_pointer);
+            DAPI_EXPECT(rocblas_status_invalid_pointer,
+                        rocblas_trsm_fn,
+                        (handle, side, uplo, transA, diag, M, N, alpha, nullptr, lda, dB, ldb));
         }
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(handle, side, uplo, transA, diag, M, N, alpha, dA, lda, nullptr, ldb),
-            rocblas_status_invalid_pointer);
+        DAPI_EXPECT(rocblas_status_invalid_pointer,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, M, N, alpha, dA, lda, nullptr, ldb));
 
         // quick return: If alpha==0, then A can be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(handle, side, uplo, transA, diag, M, N, zero, nullptr, lda, dB, ldb),
-            rocblas_status_success);
+        DAPI_EXPECT(rocblas_status_success,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, M, N, zero, nullptr, lda, dB, ldb));
 
         // quick return: If M==0, then all pointers can be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(
-                handle, side, uplo, transA, diag, 0, N, nullptr, nullptr, lda, nullptr, ldb),
-            rocblas_status_success);
+        DAPI_EXPECT(rocblas_status_success,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, 0, N, nullptr, nullptr, lda, nullptr, ldb));
 
         // quick return: If N==0, then all pointers can be nullptr without error
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(
-                handle, side, uplo, transA, diag, M, 0, nullptr, nullptr, lda, nullptr, ldb),
-            rocblas_status_success);
+        DAPI_EXPECT(rocblas_status_success,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, M, 0, nullptr, nullptr, lda, nullptr, ldb));
+    }
+}
+
+template <typename T>
+void testing_trsm_internal_interfaces(const Arguments& arg)
+{
+    // testing rocblas_internal_trsm_workspace_max_size to ensure that the sizes it gives
+    // is large enough for all/various sizes below the sizes given
+
+    int64_t M           = arg.M;
+    int64_t N           = arg.N;
+    int64_t batch_count = arg.batch_count;
+
+    rocblas_side side = char2rocblas_side(arg.side);
+
+    size_t w_x_tmp_size, w_invA_size, w_x_tmp_size_backup;
+
+    CHECK_ROCBLAS_ERROR(rocblas_internal_trsm_workspace_max_size_64<T>(
+        side, M, N, batch_count, &w_x_tmp_size, &w_invA_size, &w_x_tmp_size_backup));
+
+    // test out below for various sizes below M and N
+    for(int64_t m_smaller = M; m_smaller > 0; m_smaller--)
+    {
+        for(int64_t n_smaller = N; n_smaller > 0; n_smaller--)
+        {
+            size_t w_x_tmp_size2, w_x_tmp_arr_size2, w_invA_size2, w_invA_arr_size2,
+                w_x_tmp_size_backup2;
+
+            // This is implementation-dependent, but currently we /may/ use less memory with "skinny"
+            // matrices when transA == non-transpose.
+            // Setting this to transpose will always allocate >= non-transpose invokations, so good
+            // for this test
+            rocblas_operation transA     = rocblas_operation_transpose;
+            rocblas_status    mem_status = rocblas_internal_trsm_workspace_size<T>(
+                side,
+                transA,
+                m_smaller,
+                n_smaller,
+                batch_count, // not bothering to test smaller batch_counts
+                0, // not supporting supplied invA for max_size fn
+                &w_x_tmp_size2,
+                &w_x_tmp_arr_size2,
+                &w_invA_size2,
+                &w_invA_arr_size2,
+                &w_x_tmp_size_backup2);
+
+            if(mem_status != rocblas_status_success && mem_status != rocblas_status_continue)
+                CHECK_ROCBLAS_ERROR(mem_status);
+
+#ifdef GOOGLE_TEST
+            ASSERT_TRUE(w_x_tmp_size2 <= w_x_tmp_size && w_invA_size2 <= w_invA_size
+                        && w_x_tmp_size_backup2 <= w_x_tmp_size_backup);
+#endif
+        }
     }
 }
 
 template <typename T>
 void testing_trsm(const Arguments& arg)
 {
-    auto rocblas_trsm_fn = arg.api == FORTRAN ? rocblas_trsm<T, true> : rocblas_trsm<T, false>;
+    auto rocblas_trsm_fn = arg.api & c_API_FORTRAN ? rocblas_trsm<T, true> : rocblas_trsm<T, false>;
+    auto rocblas_trsm_fn_64
+        = arg.api & c_API_FORTRAN ? rocblas_trsm_64<T, true> : rocblas_trsm_64<T, false>;
 
-    rocblas_int M   = arg.M;
-    rocblas_int N   = arg.N;
-    rocblas_int lda = arg.lda;
-    rocblas_int ldb = arg.ldb;
+    int64_t M   = arg.M;
+    int64_t N   = arg.N;
+    int64_t lda = arg.lda;
+    int64_t ldb = arg.ldb;
 
     char char_side   = arg.side;
     char char_uplo   = arg.uplo;
@@ -221,7 +267,7 @@ void testing_trsm(const Arguments& arg)
     rocblas_operation transA = char2rocblas_operation(char_transA);
     rocblas_diagonal  diag   = char2rocblas_diagonal(char_diag);
 
-    rocblas_int K = side == rocblas_side_left ? M : N;
+    int64_t K = side == rocblas_side_left ? M : N;
 
     rocblas_local_handle handle{arg};
 
@@ -231,10 +277,9 @@ void testing_trsm(const Arguments& arg)
     {
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
 
-        EXPECT_ROCBLAS_STATUS(
-            rocblas_trsm_fn(
-                handle, side, uplo, transA, diag, M, N, nullptr, nullptr, lda, nullptr, ldb),
-            rocblas_status_invalid_size);
+        DAPI_EXPECT(rocblas_status_invalid_size,
+                    rocblas_trsm_fn,
+                    (handle, side, uplo, transA, diag, M, N, nullptr, nullptr, lda, nullptr, ldb));
 
         return;
     }
@@ -274,15 +319,13 @@ void testing_trsm(const Arguments& arg)
     copy_matrix_with_different_leading_dimensions(hX, hB);
 
     // Calculate hB = hA*hX;
-    cblas_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hB, M);
+    ref_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hB, M);
     copy_matrix_with_different_leading_dimensions(hB, hXorB_1);
 
     // copy data from CPU to device
     CHECK_HIP_ERROR(dA.transfer_from(hA));
     CHECK_HIP_ERROR(dXorB.transfer_from(hXorB_1));
 
-    double gpu_time_used, cpu_time_used;
-    gpu_time_used = cpu_time_used  = 0.0;
     double error_eps_multiplier    = ERROR_EPS_MULTIPLIER;
     double residual_eps_multiplier = RESIDUAL_EPS_MULTIPLIER;
     double eps                     = std::numeric_limits<real_t<T>>::epsilon();
@@ -293,8 +336,9 @@ void testing_trsm(const Arguments& arg)
     {
         // Compute size
         CHECK_ROCBLAS_ERROR(rocblas_start_device_memory_size_query(handle));
-        CHECK_ALLOC_QUERY(
-            rocblas_trsm_fn(handle, side, uplo, transA, diag, M, N, &alpha_h, dA, lda, dXorB, ldb));
+        DAPI_CHECK_ALLOC_QUERY(
+            rocblas_trsm_fn,
+            (handle, side, uplo, transA, diag, M, N, &alpha_h, dA, lda, dXorB, ldb));
 
         size_t size;
         CHECK_ROCBLAS_ERROR(rocblas_stop_device_memory_size_query(handle, &size));
@@ -314,16 +358,17 @@ void testing_trsm(const Arguments& arg)
             handle.pre_test(arg);
             if(arg.api != INTERNAL)
             {
-                CHECK_ROCBLAS_ERROR(rocblas_trsm_fn(
-                    handle, side, uplo, transA, diag, M, N, &alpha_h, dA, lda, dXorB, ldb));
+                DAPI_CHECK(rocblas_trsm_fn,
+                           (handle, side, uplo, transA, diag, M, N, &alpha_h, dA, lda, dXorB, ldb));
             }
             else
             {
+                // NOTE: not testing internal 64-bit API as of now
                 // internal function requires us to supply temporary memory ourselves
-                constexpr bool        BATCHED        = false;
-                constexpr rocblas_int batch_count    = 1;
-                bool                  optimal_mem    = true;
-                rocblas_int           supp_invA_size = 0; // used for trsm_ex
+                constexpr bool    BATCHED        = false;
+                constexpr int64_t batch_count    = 1;
+                bool              optimal_mem    = true;
+                int64_t           supp_invA_size = 0; // used for trsm_ex
 
                 // first exported internal interface - calculate how much mem is needed
                 size_t w_x_tmp_size, w_x_tmp_arr_size, w_invA_size, w_invA_arr_size,
@@ -403,7 +448,7 @@ void testing_trsm(const Arguments& arg)
                     trsm_err_res_check<T>(err_host, M, error_eps_multiplier, eps);
 
                 // hx_or_b contains A * (calculated X), so res = A * (calculated x) - b = hx_or_b - hb
-                cblas_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hXorB_1, ldb);
+                ref_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hXorB_1, ldb);
                 double err_host_res = matrix_norm_1<T>(M, N, hXorB_1, ldb, hB, M);
 
                 if(arg.unit_check)
@@ -423,11 +468,68 @@ void testing_trsm(const Arguments& arg)
             CHECK_HIP_ERROR(dXorB.transfer_from(hXorB_1));
             CHECK_HIP_ERROR(hipMemcpy(alpha_d, &alpha_h, sizeof(T), hipMemcpyHostToDevice));
 
-            CHECK_ROCBLAS_ERROR(rocblas_trsm_fn(
-                handle, side, uplo, transA, diag, M, N, alpha_d, dA, lda, dXorB, ldb));
+            DAPI_CHECK(rocblas_trsm_fn,
+                       (handle, side, uplo, transA, diag, M, N, alpha_d, dA, lda, dXorB, ldb));
 
             CHECK_HIP_ERROR(hXorB_1.transfer_from(dXorB));
 
+            if(arg.repeatability_check)
+            {
+                host_matrix<T> hXorB_copy(M, N, ldb);
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_matrix<T> dA_copy(K, K, lda, HMM);
+                    device_matrix<T> dXorB_copy(M, N, ldb, HMM);
+                    device_vector<T> alpha_d_copy(1);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dXorB_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(alpha_d_copy.memcheck());
+
+                    // copy data from CPU to device
+                    CHECK_HIP_ERROR(dA_copy.transfer_from(hA));
+                    CHECK_HIP_ERROR(
+                        hipMemcpy(alpha_d_copy, &alpha_h, sizeof(T), hipMemcpyHostToDevice));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        copy_matrix_with_different_leading_dimensions(hB, hXorB_copy);
+                        CHECK_HIP_ERROR(dXorB_copy.transfer_from(hXorB_copy));
+                        CHECK_ROCBLAS_ERROR(rocblas_trsm_fn(handle_copy,
+                                                            side,
+                                                            uplo,
+                                                            transA,
+                                                            diag,
+                                                            M,
+                                                            N,
+                                                            alpha_d_copy,
+                                                            dA_copy,
+                                                            lda,
+                                                            dXorB_copy,
+                                                            ldb));
+                        CHECK_HIP_ERROR(hXorB_copy.transfer_from(dXorB_copy));
+                        unit_check_general<T>(M, N, ldb, hXorB_1, hXorB_copy);
+                    }
+                }
+                return;
+            }
+
             if(alpha_h == 0)
             {
                 // expecting 0 output, set hX == 0
@@ -445,7 +547,7 @@ void testing_trsm(const Arguments& arg)
                 if(arg.unit_check)
                     trsm_err_res_check<T>(err_device, M, error_eps_multiplier, eps);
 
-                cblas_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hXorB_1, ldb);
+                ref_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA, lda, hXorB_1, ldb);
                 double err_device_res = matrix_norm_1<T>(M, N, hXorB_1, ldb, hB, M);
 
                 if(arg.unit_check)
@@ -457,28 +559,24 @@ void testing_trsm(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = arg.cold_iters;
-        int number_hot_calls  = arg.iters;
+        double gpu_time_used, cpu_time_used;
+        int    number_cold_calls = arg.cold_iters;
+        int    total_calls       = number_cold_calls + arg.iters;
 
         // GPU rocBLAS
         CHECK_HIP_ERROR(dXorB.transfer_from(hXorB_1));
-
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
 
-        for(int i = 0; i < number_cold_calls; i++)
-        {
-            CHECK_ROCBLAS_ERROR(rocblas_trsm_fn(
-                handle, side, uplo, transA, diag, M, N, &alpha_h, dA, lda, dXorB, ldb));
-        }
-
         hipStream_t stream;
         CHECK_ROCBLAS_ERROR(rocblas_get_stream(handle, &stream));
-        gpu_time_used = get_time_us_sync(stream); // in microseconds
 
-        for(int i = 0; i < number_hot_calls; i++)
+        for(int i = 0; i < total_calls; i++)
         {
-            CHECK_ROCBLAS_ERROR(rocblas_trsm_fn(
-                handle, side, uplo, transA, diag, M, N, &alpha_h, dA, lda, dXorB, ldb));
+            if(i == number_cold_calls)
+                gpu_time_used = get_time_us_sync(stream); // in microseconds
+
+            DAPI_DISPATCH(rocblas_trsm_fn,
+                          (handle, side, uplo, transA, diag, M, N, &alpha_h, dA, lda, dXorB, ldb));
         }
 
         gpu_time_used = get_time_us_sync(stream) - gpu_time_used;
@@ -487,7 +585,7 @@ void testing_trsm(const Arguments& arg)
         copy_matrix_with_different_leading_dimensions(hB, hXorB_1);
         cpu_time_used = get_time_us_no_sync();
 
-        cblas_trsm<T>(side, uplo, transA, diag, M, N, alpha_h, hA, lda, hXorB_1, ldb);
+        ref_trsm<T>(side, uplo, transA, diag, M, N, alpha_h, hA, lda, hXorB_1, ldb);
 
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
