@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,11 +28,11 @@ template <typename Tx, typename Ty = Tx, typename Tr = Ty, typename Tex = Tr, bo
 void testing_dot_batched_ex_bad_arg(const Arguments& arg)
 {
     auto rocblas_dot_batched_ex_fn
-        = arg.api == FORTRAN
+        = arg.api & c_API_FORTRAN
               ? (CONJ ? rocblas_dotc_batched_ex_fortran : rocblas_dot_batched_ex_fortran)
               : (CONJ ? rocblas_dotc_batched_ex : rocblas_dot_batched_ex);
     auto rocblas_dot_batched_ex_fn_64
-        = arg.api == FORTRAN_64
+        = arg.api & c_API_FORTRAN
               ? (CONJ ? rocblas_dotc_batched_ex_64_fortran : rocblas_dot_batched_ex_64_fortran)
               : (CONJ ? rocblas_dotc_batched_ex_64 : rocblas_dot_batched_ex_64);
 
@@ -119,21 +119,15 @@ void testing_dot_batched_ex_bad_arg(const Arguments& arg)
                  execution_type));
 }
 
-template <typename Tx, typename Ty = Tx, typename Tr = Ty, typename Tex = Tr>
-void testing_dotc_batched_ex_bad_arg(const Arguments& arg)
-{
-    testing_dot_batched_ex_bad_arg<Tx, Ty, Tr, Tex, true>(arg);
-}
-
 template <typename Tx, typename Ty = Tx, typename Tr = Ty, typename Tex = Tr, bool CONJ = false>
 void testing_dot_batched_ex(const Arguments& arg)
 {
     auto rocblas_dot_batched_ex_fn
-        = arg.api == FORTRAN
+        = arg.api & c_API_FORTRAN
               ? (CONJ ? rocblas_dotc_batched_ex_fortran : rocblas_dot_batched_ex_fortran)
               : (CONJ ? rocblas_dotc_batched_ex : rocblas_dot_batched_ex);
     auto rocblas_dot_batched_ex_fn_64
-        = arg.api == FORTRAN_64
+        = arg.api & c_API_FORTRAN
               ? (CONJ ? rocblas_dotc_batched_ex_64_fortran : rocblas_dot_batched_ex_64_fortran)
               : (CONJ ? rocblas_dotc_batched_ex_64 : rocblas_dot_batched_ex_64);
 
@@ -273,14 +267,82 @@ void testing_dot_batched_ex(const Arguments& arg)
                         result_type,
                         execution_type));
             handle.post_test(arg);
+
+            if(arg.repeatability_check)
+            {
+                host_vector<Tr> rocblas_result_device_copy(batch_count);
+                CHECK_HIP_ERROR(rocblas_result_device_copy.memcheck());
+
+                CHECK_HIP_ERROR(rocblas_result_device.transfer_from(d_rocblas_result_device));
+
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_batch_vector<Tx> dx_copy(N, incx, batch_count);
+                    device_batch_vector<Ty> dy_copy(N, incy, batch_count);
+                    device_vector<Tr>       d_rocblas_result_device_copy(batch_count);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(dy_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_rocblas_result_device_copy.memcheck());
+
+                    // copy data from CPU to device
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+                    CHECK_HIP_ERROR(dy_copy.transfer_from(hy));
+
+                    auto dy_ptr_copy
+                        = (arg.algo) ? dx_copy.ptr_on_device() : dy_copy.ptr_on_device();
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+                        DAPI_CHECK(rocblas_dot_batched_ex_fn,
+                                   (handle_copy,
+                                    N,
+                                    dx_copy.ptr_on_device(),
+                                    x_type,
+                                    incx,
+                                    dy_ptr_copy,
+                                    y_type,
+                                    incy,
+                                    batch_count,
+                                    d_rocblas_result_device_copy,
+                                    result_type,
+                                    execution_type));
+                        CHECK_HIP_ERROR(
+                            rocblas_result_device_copy.transfer_from(d_rocblas_result_device_copy));
+                        unit_check_general<Tr>(1,
+                                               1,
+                                               1,
+                                               1,
+                                               rocblas_result_device,
+                                               rocblas_result_device_copy,
+                                               batch_count);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < batch_count; ++b)
         {
-            (CONJ ? cblas_dotc<Tx>
-                  : cblas_dot<Tx>)(N, hx[b], incx, hy_ptr[b], incy, &cpu_result[b]);
+            (CONJ ? ref_dotc<Tx, Tr>
+                  : ref_dot<Tx, Tr>)(N, hx[b], incx, hy_ptr[b], incy, &cpu_result[b]);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
@@ -388,9 +450,3 @@ void testing_dot_batched_ex(const Arguments& arg)
             rocblas_error_device);
     }
 }
-
-template <typename Tx, typename Ty = Tx, typename Tr = Ty, typename Tex = Tr>
-void testing_dotc_batched_ex(const Arguments& arg)
-{
-    testing_dot_batched_ex<Tx, Ty, Tr, Tex, true>(arg);
-}
