@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,18 +24,18 @@
 
 #include "testing_common.hpp"
 
-template <typename Tx, typename Tr>
+template <typename Tx, typename Tr = Tx>
 void testing_nrm2_batched_ex_bad_arg(const Arguments& arg)
 {
     auto rocblas_nrm2_batched_ex_fn
-        = arg.api == FORTRAN ? rocblas_nrm2_batched_ex_fortran : rocblas_nrm2_batched_ex;
+        = arg.api & c_API_FORTRAN ? rocblas_nrm2_batched_ex_fortran : rocblas_nrm2_batched_ex;
 
     auto rocblas_nrm2_batched_ex_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_nrm2_batched_ex_64_fortran : rocblas_nrm2_batched_ex_64;
+        = arg.api & c_API_FORTRAN ? rocblas_nrm2_batched_ex_64_fortran : rocblas_nrm2_batched_ex_64;
 
-    rocblas_datatype x_type         = rocblas_datatype_f32_r;
-    rocblas_datatype result_type    = rocblas_datatype_f32_r;
-    rocblas_datatype execution_type = rocblas_datatype_f32_r;
+    rocblas_datatype x_type         = rocblas_type2datatype<Tx>();
+    rocblas_datatype result_type    = rocblas_type2datatype<Tr>();
+    rocblas_datatype execution_type = arg.compute_type;
 
     int64_t N           = 100;
     int64_t incx        = 1;
@@ -89,17 +89,17 @@ void testing_nrm2_batched_ex_bad_arg(const Arguments& arg)
                  execution_type));
 }
 
-template <typename Tx, typename Tr>
+template <typename Tx, typename Tr = Tx>
 void testing_nrm2_batched_ex(const Arguments& arg)
 {
     auto rocblas_nrm2_batched_ex_fn
-        = arg.api == FORTRAN ? rocblas_nrm2_batched_ex_fortran : rocblas_nrm2_batched_ex;
+        = arg.api & c_API_FORTRAN ? rocblas_nrm2_batched_ex_fortran : rocblas_nrm2_batched_ex;
 
     auto rocblas_nrm2_batched_ex_fn_64
-        = arg.api == FORTRAN_64 ? rocblas_nrm2_batched_ex_64_fortran : rocblas_nrm2_batched_ex_64;
+        = arg.api & c_API_FORTRAN ? rocblas_nrm2_batched_ex_64_fortran : rocblas_nrm2_batched_ex_64;
 
-    rocblas_datatype x_type         = arg.a_type;
-    rocblas_datatype result_type    = arg.b_type;
+    rocblas_datatype x_type         = rocblas_type2datatype<Tx>();
+    rocblas_datatype result_type    = rocblas_type2datatype<Tr>();
     rocblas_datatype execution_type = arg.compute_type;
 
     int64_t N           = arg.N;
@@ -219,13 +219,62 @@ void testing_nrm2_batched_ex(const Arguments& arg)
                         result_type,
                         execution_type));
             handle.post_test(arg);
+            if(arg.repeatability_check)
+            {
+                host_vector<Tr> rocblas_result_copy(batch_count);
+                CHECK_HIP_ERROR(rocblas_result_copy.memcheck());
+                CHECK_HIP_ERROR(rocblas_result_2.transfer_from(d_rocblas_result_2));
+                // multi-GPU support
+                int device_id, device_count;
+                CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+                for(int dev_id = 0; dev_id < device_count; dev_id++)
+                {
+                    CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                    if(device_id != dev_id)
+                        CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                    //New rocblas handle for new device
+                    rocblas_local_handle handle_copy{arg};
+
+                    //Allocate device memory in new device
+                    device_batch_vector<Tx> dx_copy(N, incx, batch_count);
+                    device_vector<Tr>       d_rocblas_result_2_copy(batch_count);
+
+                    // Check device memory allocation
+                    CHECK_DEVICE_ALLOCATION(dx_copy.memcheck());
+                    CHECK_DEVICE_ALLOCATION(d_rocblas_result_2_copy.memcheck());
+                    CHECK_HIP_ERROR(dx_copy.transfer_from(hx));
+
+                    CHECK_ROCBLAS_ERROR(
+                        rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                    for(int runs = 0; runs < arg.iters; runs++)
+                    {
+
+                        DAPI_CHECK(rocblas_nrm2_batched_ex_fn,
+                                   (handle_copy,
+                                    N,
+                                    dx_copy.ptr_on_device(),
+                                    x_type,
+                                    incx,
+                                    batch_count,
+                                    d_rocblas_result_2_copy,
+                                    result_type,
+                                    execution_type));
+                        CHECK_HIP_ERROR(rocblas_result_copy.transfer_from(d_rocblas_result_2_copy));
+                        unit_check_general<Tr, Tr>(
+                            batch_count, 1, 1, rocblas_result_2, rocblas_result_copy);
+                    }
+                }
+                return;
+            }
         }
 
         // CPU BLAS
         cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < batch_count; b++)
         {
-            cblas_nrm2<Tx>(N, hx[b], incx, cpu_result + b);
+            ref_nrm2<Tx>(N, hx[b], incx, cpu_result + b);
         }
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
