@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
 #pragma once
 
 #include "cblas_interface.hpp"
+#include "client_utility.hpp"
 #include "flops.hpp"
 #include "norm.hpp"
 #include "rocblas.hpp"
@@ -34,7 +35,6 @@
 #include "rocblas_test.hpp"
 #include "rocblas_vector.hpp"
 #include "unit.hpp"
-#include "utility.hpp"
 
 #define ERROR_EPS_MULTIPLIER 40
 #define RESIDUAL_EPS_MULTIPLIER 40
@@ -43,7 +43,7 @@
 template <typename T>
 void testing_trsm_strided_batched_ex_bad_arg(const Arguments& arg)
 {
-    auto rocblas_trsm_strided_batched_ex_fn = arg.api == FORTRAN
+    auto rocblas_trsm_strided_batched_ex_fn = arg.api & c_API_FORTRAN
                                                   ? rocblas_trsm_strided_batched_ex_fortran
                                                   : rocblas_trsm_strided_batched_ex;
 
@@ -296,7 +296,7 @@ void testing_trsm_strided_batched_ex_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_trsm_strided_batched_ex(const Arguments& arg)
 {
-    auto rocblas_trsm_strided_batched_ex_fn = arg.api == FORTRAN
+    auto rocblas_trsm_strided_batched_ex_fn = arg.api & c_API_FORTRAN
                                                   ? rocblas_trsm_strided_batched_ex_fortran
                                                   : rocblas_trsm_strided_batched_ex;
 
@@ -361,6 +361,7 @@ void testing_trsm_strided_batched_ex(const Arguments& arg)
     host_strided_batch_matrix<T> hX(M, N, ldb, stride_B, batch_count);
     host_strided_batch_matrix<T> hXorB_1(M, N, ldb, stride_B, batch_count);
     host_strided_batch_matrix<T> hXorB_2(M, N, ldb, stride_B, batch_count);
+    host_strided_batch_matrix<T> invATemp1(TRSM_BLOCK, TRSM_BLOCK, K, stride_invA, batch_count);
     host_strided_batch_matrix<T> cpuXorB(M, N, ldb, stride_B, batch_count);
 
     // Check host memory allocation
@@ -406,17 +407,17 @@ void testing_trsm_strided_batched_ex(const Arguments& arg)
 
     // Calculate hB = hA*hX;
     for(int b = 0; b < batch_count; b++)
-        cblas_trmm<T>(side,
-                      uplo,
-                      transA,
-                      diag,
-                      M,
-                      N,
-                      1.0 / alpha_h,
-                      hA + b * stride_A,
-                      lda,
-                      hB + b * stride_B,
-                      ldb);
+        ref_trmm<T>(side,
+                    uplo,
+                    transA,
+                    diag,
+                    M,
+                    N,
+                    1.0 / alpha_h,
+                    hA + b * stride_A,
+                    lda,
+                    hB + b * stride_B,
+                    ldb);
 
     hXorB_1.copy_from(hB);
     hXorB_2.copy_from(hB);
@@ -510,6 +511,7 @@ void testing_trsm_strided_batched_ex(const Arguments& arg)
 
     if(arg.unit_check || arg.norm_check)
     {
+
         // calculate dXorB <- A^(-1) B   rocblas_device_pointer_host
         CHECK_ROCBLAS_ERROR(rocblas_set_pointer_mode(handle, rocblas_pointer_mode_host));
         CHECK_HIP_ERROR(dXorB.transfer_from(hXorB_1));
@@ -602,6 +604,75 @@ void testing_trsm_strided_batched_ex(const Arguments& arg)
 
         CHECK_HIP_ERROR(hXorB_2.transfer_from(dXorB));
 
+        if(arg.repeatability_check)
+        {
+            host_strided_batch_matrix<T> hXorB_copy(M, N, ldb, stride_B, batch_count);
+            CHECK_HIP_ERROR(hXorB_copy.memcheck());
+            CHECK_HIP_ERROR(invATemp1.transfer_from(dinvA));
+            // multi-GPU support
+            int device_id, device_count;
+            CHECK_HIP_ERROR(hipGetDeviceCount(&device_count));
+            for(int dev_id = 0; dev_id < device_count; dev_id++)
+            {
+                CHECK_HIP_ERROR(hipGetDevice(&device_id));
+                if(device_id != dev_id)
+                    CHECK_HIP_ERROR(hipSetDevice(dev_id));
+
+                //New rocblas handle for new device
+                rocblas_local_handle handle_copy{arg};
+
+                //Allocate device memory in new device
+                device_strided_batch_matrix<T> dA_copy(K, K, lda, stride_A, batch_count);
+                device_strided_batch_matrix<T> dXorB_copy(M, N, ldb, stride_B, batch_count);
+                device_strided_batch_matrix<T> dinvA_copy(
+                    TRSM_BLOCK, TRSM_BLOCK, K, stride_invA, batch_count);
+                device_vector<T> alpha_d_copy(1);
+
+                // Check device memory allocation
+                CHECK_DEVICE_ALLOCATION(dA_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dXorB_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(dinvA_copy.memcheck());
+                CHECK_DEVICE_ALLOCATION(alpha_d_copy.memcheck());
+
+                CHECK_HIP_ERROR(dA_copy.transfer_from(hA));
+                CHECK_HIP_ERROR(dinvA_copy.transfer_from(invATemp1));
+
+                CHECK_HIP_ERROR(
+                    hipMemcpy(alpha_d_copy, &alpha_h, sizeof(T), hipMemcpyHostToDevice));
+
+                CHECK_ROCBLAS_ERROR(
+                    rocblas_set_pointer_mode(handle_copy, rocblas_pointer_mode_device));
+
+                for(int runs = 0; runs < arg.iters; runs++)
+                {
+                    CHECK_HIP_ERROR(dXorB_copy.transfer_from(hB));
+                    CHECK_ROCBLAS_ERROR(rocblas_trsm_strided_batched_ex_fn(handle_copy,
+                                                                           side,
+                                                                           uplo,
+                                                                           transA,
+                                                                           diag,
+                                                                           M,
+                                                                           N,
+                                                                           alpha_d_copy,
+                                                                           dA_copy,
+                                                                           lda,
+                                                                           stride_A,
+                                                                           dXorB_copy,
+                                                                           ldb,
+                                                                           stride_B,
+                                                                           batch_count,
+                                                                           dinvA_copy,
+                                                                           size_invA,
+                                                                           stride_invA,
+                                                                           arg.compute_type));
+
+                    CHECK_HIP_ERROR(hXorB_copy.transfer_from(dXorB_copy));
+                    unit_check_general<T>(M, N, ldb, stride_B, hXorB_2, hXorB_copy, batch_count);
+                }
+            }
+            return;
+        }
+
         //computed result is in hx_or_b, so forward error is E = hx - hx_or_b
         // calculate vector-induced-norm 1 of matrix E
         for(int b = 0; b < batch_count; b++)
@@ -614,10 +685,8 @@ void testing_trsm_strided_batched_ex(const Arguments& arg)
             trsm_err_res_check<T>(max_err_2, M, error_eps_multiplier, eps);
 
             // hx_or_b contains A * (calculated X), so res = A * (calculated x) - b = hx_or_b - hb
-            cblas_trmm<T>(
-                side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA[b], lda, hXorB_1[b], ldb);
-            cblas_trmm<T>(
-                side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA[b], lda, hXorB_2[b], ldb);
+            ref_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA[b], lda, hXorB_1[b], ldb);
+            ref_trmm<T>(side, uplo, transA, diag, M, N, 1.0 / alpha_h, hA[b], lda, hXorB_2[b], ldb);
 
             // calculate vector-induced-norm 1 of matrix res
             max_err_1 = rocblas_abs(matrix_norm_1<T>(M, N, ldb, hXorB_1[b], hB[b]));
@@ -695,7 +764,7 @@ void testing_trsm_strided_batched_ex(const Arguments& arg)
         cpu_time_used = get_time_us_no_sync();
 
         for(int b = 0; b < batch_count; b++)
-            cblas_trsm<T>(side, uplo, transA, diag, M, N, alpha_h, hA[b], lda, cpuXorB[b], ldb);
+            ref_trsm<T>(side, uplo, transA, diag, M, N, alpha_h, hA[b], lda, cpuXorB[b], ldb);
 
         cpu_time_used = get_time_us_no_sync() - cpu_time_used;
 
