@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -20,45 +20,45 @@
  *
  * ************************************************************************ */
 
-#include "check_numerics_vector.hpp"
-#include "handle.hpp"
-#include "rocblas.h"
 #include "rocblas_syr2.hpp"
 
-template <typename T>
-__device__ void rocblas_syr2_kernel_calc(bool        is_upper,
-                                         rocblas_int n,
-                                         T           alpha,
-                                         const T*    x,
-                                         rocblas_int incx,
-                                         const T*    y,
-                                         rocblas_int incy,
-                                         T*          A,
-                                         rocblas_int lda)
+template <int DIM_X, int DIM_Y, int N_TX, typename T>
+__device__ void rocblas_syr2_kernel_calc(bool     is_upper,
+                                         int      n,
+                                         T        alpha,
+                                         const T* x,
+                                         int64_t  incx,
+                                         const T* y,
+                                         int64_t  incy,
+                                         T*       A,
+                                         int64_t  lda)
 {
-    rocblas_int tx = blockIdx.x * blockDim.x + threadIdx.x;
-    rocblas_int ty = blockIdx.y * blockDim.y + threadIdx.y;
-
-    if(is_upper ? ty < n && tx <= ty : tx < n && ty <= tx)
-        A[tx + ty * int64_t(lda)] += alpha * x[tx * int64_t(incx)] * y[ty * int64_t(incy)]
-                                     + alpha * y[tx * int64_t(incy)] * x[ty * int64_t(incx)];
+    int tx = (blockIdx.x * DIM_X * N_TX) + threadIdx.x;
+    int ty = blockIdx.y * DIM_Y + threadIdx.y;
+#pragma unroll
+    for(int i = 0; i < N_TX; i++, tx += DIM_X)
+    {
+        if(is_upper ? ty < n && tx <= ty : tx < n && ty <= tx)
+            A[tx + ty * lda]
+                += alpha * x[tx * incx] * y[ty * incy] + alpha * y[tx * incy] * x[ty * incx];
+    }
 }
 
-template <rocblas_int DIM_X, rocblas_int DIM_Y, typename TScal, typename TConstPtr, typename TPtr>
+template <int DIM_X, int DIM_Y, int N_TX, typename TScal, typename TConstPtr, typename TPtr>
 ROCBLAS_KERNEL(DIM_X* DIM_Y)
 rocblas_syr2_kernel(bool           is_upper,
                     rocblas_int    n,
                     TScal          alphaa,
                     TConstPtr      xa,
                     rocblas_stride shift_x,
-                    rocblas_int    incx,
+                    int64_t        incx,
                     rocblas_stride stride_x,
                     TConstPtr      ya,
                     rocblas_stride shift_y,
-                    rocblas_int    incy,
+                    int64_t        incy,
                     rocblas_stride stride_y,
                     TPtr           Aa,
-                    rocblas_int    lda,
+                    int64_t        lda,
                     rocblas_stride shift_A,
                     rocblas_stride stride_A)
 {
@@ -74,7 +74,7 @@ rocblas_syr2_kernel(bool           is_upper,
     const auto* x = load_ptr_batch(xa, blockIdx.z, shift_x, stride_x);
     const auto* y = load_ptr_batch(ya, blockIdx.z, shift_y, stride_y);
 
-    rocblas_syr2_kernel_calc(is_upper, n, alpha, x, incx, y, incy, A, lda);
+    rocblas_syr2_kernel_calc<DIM_X, DIM_Y, N_TX>(is_upper, n, alpha, x, incx, y, incy, A, lda);
 }
 
 /**
@@ -84,20 +84,20 @@ rocblas_syr2_kernel(bool           is_upper,
  * Where T is the bast type (float or double)
  */
 template <typename TScal, typename TConstPtr, typename TPtr>
-rocblas_status rocblas_internal_syr2_template(rocblas_handle handle,
+rocblas_status rocblas_internal_syr2_launcher(rocblas_handle handle,
                                               rocblas_fill   uplo,
                                               rocblas_int    n,
                                               TScal          alpha,
                                               TConstPtr      x,
                                               rocblas_stride offset_x,
-                                              rocblas_int    incx,
+                                              int64_t        incx,
                                               rocblas_stride stride_x,
                                               TConstPtr      y,
                                               rocblas_stride offset_y,
-                                              rocblas_int    incy,
+                                              int64_t        incy,
                                               rocblas_stride stride_y,
                                               TPtr           A,
-                                              rocblas_int    lda,
+                                              int64_t        lda,
                                               rocblas_stride offset_A,
                                               rocblas_stride stride_A,
                                               rocblas_int    batch_count)
@@ -107,19 +107,23 @@ rocblas_status rocblas_internal_syr2_template(rocblas_handle handle,
         return rocblas_status_success;
 
     // in case of negative inc, shift pointer to end of data for negative indexing tid*inc
-    int64_t shift_x = incx < 0 ? offset_x - int64_t(incx) * (n - 1) : offset_x;
-    int64_t shift_y = incy < 0 ? offset_y - int64_t(incy) * (n - 1) : offset_y;
+    int64_t shift_x = incx < 0 ? offset_x - incx * (n - 1) : offset_x;
+    int64_t shift_y = incy < 0 ? offset_y - incy * (n - 1) : offset_y;
+
+    //Identifying the precision to have an appropriate optimization
+    static constexpr bool is_float = std::is_same_v<TScal, const float*>;
 
     static constexpr int SYR2_DIM_X = 128;
     static constexpr int SYR2_DIM_Y = 8;
-    rocblas_int          blocksX    = (n - 1) / SYR2_DIM_X + 1;
+    static constexpr int N_TX       = is_float ? 2 : 1; // x items per x thread
+    rocblas_int          blocksX    = (n - 1) / (SYR2_DIM_X * N_TX) + 1;
     rocblas_int          blocksY    = (n - 1) / SYR2_DIM_Y + 1;
 
     dim3 syr2_grid(blocksX, blocksY, batch_count);
     dim3 syr2_threads(SYR2_DIM_X, SYR2_DIM_Y);
 
     if(rocblas_pointer_mode_device == handle->pointer_mode)
-        ROCBLAS_LAUNCH_KERNEL((rocblas_syr2_kernel<SYR2_DIM_X, SYR2_DIM_Y>),
+        ROCBLAS_LAUNCH_KERNEL((rocblas_syr2_kernel<SYR2_DIM_X, SYR2_DIM_Y, N_TX>),
                               syr2_grid,
                               syr2_threads,
                               0,
@@ -140,7 +144,7 @@ rocblas_status rocblas_internal_syr2_template(rocblas_handle handle,
                               offset_A,
                               stride_A);
     else
-        ROCBLAS_LAUNCH_KERNEL((rocblas_syr2_kernel<SYR2_DIM_X, SYR2_DIM_Y>),
+        ROCBLAS_LAUNCH_KERNEL((rocblas_syr2_kernel<SYR2_DIM_X, SYR2_DIM_Y, N_TX>),
                               syr2_grid,
                               syr2_threads,
                               0,
@@ -184,23 +188,23 @@ ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
                                    rocblas_stride stride_A,
                                    rocblas_int    batch_count)
 {
-    return rocblas_internal_syr2_template<const T*>(handle,
-                                                    uplo,
-                                                    n,
-                                                    alpha,
-                                                    x,
-                                                    offset_x,
-                                                    incx,
-                                                    stride_x,
-                                                    y,
-                                                    offset_y,
-                                                    incy,
-                                                    stride_y,
-                                                    A,
-                                                    lda,
-                                                    offset_A,
-                                                    stride_A,
-                                                    batch_count);
+    return rocblas_internal_syr2_launcher(handle,
+                                          uplo,
+                                          n,
+                                          alpha,
+                                          x,
+                                          offset_x,
+                                          incx,
+                                          stride_x,
+                                          y,
+                                          offset_y,
+                                          incy,
+                                          stride_y,
+                                          A,
+                                          lda,
+                                          offset_A,
+                                          stride_A,
+                                          batch_count);
 }
 
 template <typename T>
@@ -223,7 +227,7 @@ ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status
                                            rocblas_stride  stride_A,
                                            rocblas_int     batch_count)
 {
-    return rocblas_internal_syr2_template(handle,
+    return rocblas_internal_syr2_launcher(handle,
                                           uplo,
                                           n,
                                           alpha,
@@ -246,20 +250,20 @@ template <typename T, typename U>
 rocblas_status rocblas_syr2_check_numerics(const char*    function_name,
                                            rocblas_handle handle,
                                            rocblas_fill   uplo,
-                                           rocblas_int    n,
+                                           int64_t        n,
                                            T              A,
                                            rocblas_stride offset_a,
-                                           rocblas_int    lda,
+                                           int64_t        lda,
                                            rocblas_stride stride_a,
                                            U              x,
                                            rocblas_stride offset_x,
-                                           rocblas_int    inc_x,
+                                           int64_t        inc_x,
                                            rocblas_stride stride_x,
                                            U              y,
                                            rocblas_stride offset_y,
-                                           rocblas_int    inc_y,
+                                           int64_t        inc_y,
                                            rocblas_stride stride_y,
-                                           rocblas_int    batch_count,
+                                           int64_t        batch_count,
                                            const int      check_numerics,
                                            bool           is_input)
 {
@@ -315,32 +319,29 @@ rocblas_status rocblas_syr2_check_numerics(const char*    function_name,
 // Instantiations below will need to be manually updated to match any change in
 // template parameters in the files *syr2*.cpp
 
-// clang-format off
-
 #ifdef INSTANTIATE_SYR2_TEMPLATE
 #error INSTANTIATE_SYR2_TEMPLATE already defined
 #endif
 
-#define INSTANTIATE_SYR2_TEMPLATE(T_)    \
-template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status        \
-    rocblas_internal_syr2_template<T_>                          \
-                                  (rocblas_handle handle,       \
-                                   rocblas_fill   uplo,         \
-                                   rocblas_int    n,            \
-                                   const T_*      alpha,        \
-                                   const T_*      x,            \
-                                   rocblas_stride offset_x,     \
-                                   rocblas_int    incx,         \
-                                   rocblas_stride stride_x,     \
-                                   const T_*      y,            \
-                                   rocblas_stride offset_y,     \
-                                   rocblas_int    incy,         \
-                                   rocblas_stride stride_y,     \
-                                   T_*            A,            \
-                                   rocblas_int    lda,          \
-                                   rocblas_stride offset_A,     \
-                                   rocblas_stride stride_A,     \
-                                   rocblas_int    batch_count);
+#define INSTANTIATE_SYR2_TEMPLATE(T_)                                                            \
+    template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status rocblas_internal_syr2_template<T_>( \
+        rocblas_handle handle,                                                                   \
+        rocblas_fill   uplo,                                                                     \
+        rocblas_int    n,                                                                        \
+        const T_*      alpha,                                                                    \
+        const T_*      x,                                                                        \
+        rocblas_stride offset_x,                                                                 \
+        rocblas_int    incx,                                                                     \
+        rocblas_stride stride_x,                                                                 \
+        const T_*      y,                                                                        \
+        rocblas_stride offset_y,                                                                 \
+        rocblas_int    incy,                                                                     \
+        rocblas_stride stride_y,                                                                 \
+        T_*            A,                                                                        \
+        rocblas_int    lda,                                                                      \
+        rocblas_stride offset_A,                                                                 \
+        rocblas_stride stride_A,                                                                 \
+        rocblas_int    batch_count);
 
 INSTANTIATE_SYR2_TEMPLATE(float)
 INSTANTIATE_SYR2_TEMPLATE(double)
@@ -353,26 +354,25 @@ INSTANTIATE_SYR2_TEMPLATE(rocblas_double_complex)
 #error INSTANTIATE_SYR2_BATCHED_TEMPLATE already defined
 #endif
 
-#define INSTANTIATE_SYR2_BATCHED_TEMPLATE(T_)    \
-template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status         \
-    rocblas_internal_syr2_batched_template<T_>                   \
-                                  (rocblas_handle   handle,       \
-                                   rocblas_fill     uplo,         \
-                                   rocblas_int      n,            \
-                                   const T_*        alpha,        \
-                                   const T_* const* x,            \
-                                   rocblas_stride   offset_x,     \
-                                   rocblas_int      incx,         \
-                                   rocblas_stride   stride_x,     \
-                                   const T_* const* y,            \
-                                   rocblas_stride   offset_y,     \
-                                   rocblas_int      incy,         \
-                                   rocblas_stride   stride_y,     \
-                                   T_* const*       A,            \
-                                   rocblas_int      lda,          \
-                                   rocblas_stride   offset_A,     \
-                                   rocblas_stride   stride_A,     \
-                                   rocblas_int      batch_count);
+#define INSTANTIATE_SYR2_BATCHED_TEMPLATE(T_)                                 \
+    template ROCBLAS_INTERNAL_EXPORT_NOINLINE rocblas_status                  \
+        rocblas_internal_syr2_batched_template<T_>(rocblas_handle   handle,   \
+                                                   rocblas_fill     uplo,     \
+                                                   rocblas_int      n,        \
+                                                   const T_*        alpha,    \
+                                                   const T_* const* x,        \
+                                                   rocblas_stride   offset_x, \
+                                                   rocblas_int      incx,     \
+                                                   rocblas_stride   stride_x, \
+                                                   const T_* const* y,        \
+                                                   rocblas_stride   offset_y, \
+                                                   rocblas_int      incy,     \
+                                                   rocblas_stride   stride_y, \
+                                                   T_* const*       A,        \
+                                                   rocblas_int      lda,      \
+                                                   rocblas_stride   offset_A, \
+                                                   rocblas_stride   stride_A, \
+                                                   rocblas_int      batch_count);
 
 INSTANTIATE_SYR2_BATCHED_TEMPLATE(float)
 INSTANTIATE_SYR2_BATCHED_TEMPLATE(double)
@@ -385,27 +385,26 @@ INSTANTIATE_SYR2_BATCHED_TEMPLATE(rocblas_double_complex)
 #error INSTANTIATE_SYR2_NUMERICS already defined
 #endif
 
-#define INSTANTIATE_SYR2_NUMERICS(T_, U_)                                 \
-template rocblas_status rocblas_syr2_check_numerics<T_, U_>               \
-                                          (const char*    function_name,  \
-                                           rocblas_handle handle,         \
-                                           rocblas_fill   uplo,           \
-                                           rocblas_int    n,              \
-                                           T_             A,              \
-                                           rocblas_stride    offset_a,    \
-                                           rocblas_int    lda,            \
-                                           rocblas_stride stride_a,       \
-                                           U_             x,              \
-                                           rocblas_stride    offset_x,    \
-                                           rocblas_int    inc_x,          \
-                                           rocblas_stride stride_x,       \
-                                           U_             y,              \
-                                           rocblas_stride    offset_y,    \
-                                           rocblas_int    inc_y,          \
-                                           rocblas_stride stride_y,       \
-                                           rocblas_int    batch_count,    \
-                                           const int      check_numerics, \
-                                           bool           is_input);
+#define INSTANTIATE_SYR2_NUMERICS(T_, U_)                                                      \
+    template rocblas_status rocblas_syr2_check_numerics<T_, U_>(const char*    function_name,  \
+                                                                rocblas_handle handle,         \
+                                                                rocblas_fill   uplo,           \
+                                                                int64_t        n,              \
+                                                                T_             A,              \
+                                                                rocblas_stride offset_a,       \
+                                                                int64_t        lda,            \
+                                                                rocblas_stride stride_a,       \
+                                                                U_             x,              \
+                                                                rocblas_stride offset_x,       \
+                                                                int64_t        inc_x,          \
+                                                                rocblas_stride stride_x,       \
+                                                                U_             y,              \
+                                                                rocblas_stride offset_y,       \
+                                                                int64_t        inc_y,          \
+                                                                rocblas_stride stride_y,       \
+                                                                int64_t        batch_count,    \
+                                                                const int      check_numerics, \
+                                                                bool           is_input);
 
 INSTANTIATE_SYR2_NUMERICS(float*, float const*)
 INSTANTIATE_SYR2_NUMERICS(double*, double const*)
@@ -417,5 +416,3 @@ INSTANTIATE_SYR2_NUMERICS(rocblas_float_complex* const*, rocblas_float_complex c
 INSTANTIATE_SYR2_NUMERICS(rocblas_double_complex* const*, rocblas_double_complex const* const*)
 
 #undef INSTANTIATE_SYR2_NUMERICS
-
-// clang-format on
