@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2016-2023 Advanced Micro Devices, Inc. All rights reserved.
+* Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,9 +23,9 @@
 #pragma once
 
 #include "check_numerics_matrix.hpp"
-#include "gemm.hpp"
 #include "handle.hpp"
 #include "rocblas_block_sizes.h"
+#include "rocblas_gemm.hpp"
 
 template <typename U, typename V>
 inline rocblas_status rocblas_trtri_arg_check(rocblas_handle   handle,
@@ -38,6 +38,10 @@ inline rocblas_status rocblas_trtri_arg_check(rocblas_handle   handle,
                                               rocblas_int      ldinvA,
                                               rocblas_int      batch_count)
 {
+    if(batch_count > c_YZ_grid_launch_limit && handle->isYZGridDim16bit())
+    {
+        return rocblas_status_invalid_size;
+    }
 
     if(uplo != rocblas_fill_lower && uplo != rocblas_fill_upper)
         return rocblas_status_invalid_value;
@@ -670,54 +674,54 @@ rocblas_status rocblas_trtri_gemm_block(rocblas_handle handle,
         }
 
         // We are naively iterating through the batches, and uses sub-batches in a strided_batched style.
-        status = rocblas_internal_gemm_template<false>(handle,
-                                                       rocblas_operation_none,
-                                                       rocblas_operation_none,
-                                                       M,
-                                                       N,
-                                                       N,
-                                                       &one,
-                                                       aptr,
-                                                       0,
-                                                       ld_A,
-                                                       sub_stride_A,
-                                                       invAg1ptr,
-                                                       0,
-                                                       ld_invA,
-                                                       sub_stride_invA,
-                                                       &zero,
-                                                       cptr,
-                                                       0,
-                                                       ld_C,
-                                                       sub_stride_C,
-                                                       sub_blocks);
+        status = rocblas_internal_gemm<false>(handle,
+                                              rocblas_operation_none,
+                                              rocblas_operation_none,
+                                              M,
+                                              N,
+                                              N,
+                                              &one,
+                                              aptr,
+                                              0,
+                                              ld_A,
+                                              sub_stride_A,
+                                              invAg1ptr,
+                                              0,
+                                              ld_invA,
+                                              sub_stride_invA,
+                                              &zero,
+                                              cptr,
+                                              0,
+                                              ld_C,
+                                              sub_stride_C,
+                                              sub_blocks);
 
         if(status != rocblas_status_success)
             break;
 
         // second batched gemm compute  invA21 = -invA22 * C (lower) or invA12 = -invA11*C (upper)
         // distance between each invA21 or invA12 is stride_invA,
-        status = rocblas_internal_gemm_template<false>(handle,
-                                                       rocblas_operation_none,
-                                                       rocblas_operation_none,
-                                                       M,
-                                                       N,
-                                                       M,
-                                                       &negative_one,
-                                                       invAg2ptr,
-                                                       0,
-                                                       ld_invA,
-                                                       sub_stride_invA,
-                                                       (const T*)cptr,
-                                                       0,
-                                                       ld_C,
-                                                       sub_stride_C,
-                                                       &zero,
-                                                       invAg2cptr,
-                                                       0,
-                                                       ld_invA,
-                                                       sub_stride_invA,
-                                                       sub_blocks);
+        status = rocblas_internal_gemm<false>(handle,
+                                              rocblas_operation_none,
+                                              rocblas_operation_none,
+                                              M,
+                                              N,
+                                              M,
+                                              &negative_one,
+                                              invAg2ptr,
+                                              0,
+                                              ld_invA,
+                                              sub_stride_invA,
+                                              (const T*)cptr,
+                                              0,
+                                              ld_C,
+                                              sub_stride_C,
+                                              &zero,
+                                              invAg2cptr,
+                                              0,
+                                              ld_invA,
+                                              sub_stride_invA,
+                                              sub_blocks);
         if(status != rocblas_status_success)
             break;
     }
